# coding: latin-1
"""
Routinen fr die Erzeugung des amtlichen Datensatzes
fr die Bundesjugendhilfestatistik

Die Funktion jghexport(jahr) gibt zwei Strings zurck:

- den abzugebenden Datensatz ohne Zeilenumbrche
- den Kontrolldatensatz mit Zeilenumbrchen und der zustzlichen
  Angabe der Fallnummer und der Mitarbeiterkennung pro Fall

Die Funktion wird vom Webinterface (html/datenaustausch.py) benutzt, um
dem Anwender zu ermglichen, den abzugebenen Datensatz und den
Kontrolldatensatz herunterzuladen.

Auerdem kann diese Datei als Skript verwendet werden, um die Datenstze
direkt als Dateien ohne Umweg ber das Webinterface zu erzeugen.
Dazu ist diese Datei (jghexport.py) in das Verzeichnis der Instanz
zu kopieren, fr die die Statistik erzeugt werden soll.

Aufruf:

python jghexport.py <jahr>

python sollte zum Aufruf desselben Interpreters fhren, mit dem EBKuS
installiert wurde.
"""


if __name__ == '__main__':
    import sys
    try:
        import init
    except ImportError:
        print "Dieses Skript zuerst in das Verzeichnis der Instanz kopieren,"
        print "fuer die der JGH-Export durchgefuehrt werden soll."
        sys.exit(1)
        
from ebkus.config import config
from ebkus.app.ebapi import JugendhilfestatistikList, Jugendhilfestatistik2007List, today, EE
from ebkus.app.ebupd import jgh_laufende_nummer_setzen
from ebkus.html.jghstatistik import get_ags_plz_ort_if_wohnort_ausserhalb

## def jghexport(jahr):
##     """Erzeugt den Datensatz fr die Jugendhilfestatistik.

##     Es werden zwei Strings zurckgegeben, der eigentliche Datensatz
##     und ein Kontrolldatensatz mit Zeilenumbrchen und vorangesteller
##     Fallnummer und Mitarbeiterkennung.
##     """
##     import time
##     from ebkus.app.ebapi import JugendhilfestatistikList
##     from ebkus.app.ebupd import jgh_laufende_nummer_setzen
##     jgh_laufende_nummer_setzen()
##     jghl = JugendhilfestatistikList (where = 'ey =  %s' % jahr, order = 'id')
##     daten_saetze = []
##     log_daten_saetze = []
##     log_header = """Erzeugen der Datei mit den Datenstzen fr die Jugendhilfestatistik fr %s

## Datum: %s
## Datei: jgh_%s.txt
## Anzahl der Datenstze: %s

## Datenstze (vorangestellt zustzlich Fallnummer und Mitarbeiterkennung):

## """ % (jahr, time.strftime("%c", time.localtime(time.time())),
##        jahr, len(jghl))
##     for record in jghl:
##         daten_satz, log_daten_satz = get_datensatz(record)
##         daten_saetze.append(daten_satz)
##         log_daten_saetze.append(log_daten_satz)
##     return ''.join(daten_saetze), log_header + ''.join(log_daten_saetze)

def check_gef8a(jgh_list):
    """Diese Prfung wird hier gemacht, weil im Jahre 2012 mglicherweise
Flle bereits abgeschlossen wurden inklusive Bundesstatistik bevor dieses Update (EBKuS 4.5)
mit der neuen Frage bzgl. der Gefhrdungseinschtzung installiert war."""
    fallnummern_ohne_gef8a = []
    for jgh in jgh_list:
        #print 'CHECK_GEF8A:', jgh.get('gef8a')
        if not jgh.get('gef8a'):
            fallnummern_ohne_gef8a.append(jgh.get('fall_fn'))
    if fallnummern_ohne_gef8a:
        raise EE("Fr folgende Fallnummern fehlt Angabe "
"\"Einleitung der Hilfe auf Grund einer vorangegangenen "
"Gefhrdungseinschtzung gem  8a Absatz 1 SGB VIII\": %s"
% ', '.join(fallnummern_ohne_gef8a))


def jghexport(jahr, andauernd):
    """Erzeugt den Datensatz fr die Jugendhilfestatistik.

    Es werden zwei Strings zurckgegeben, der eigentliche Datensatz
    und ein Kontrolldatensatz mit Zeilenumbrchen und vorangesteller
    Fallnummer und Mitarbeiterkennung.
    """
    jgh_laufende_nummer_setzen()
    if jahr >= 2007:
        if jahr >= 2016:
            get_datensatz = get_datensatz_ab_2016
        else:
            get_datensatz = get_datensatz_ab_2007
        if andauernd:
            #jghl = Jugendhilfestatistik2007List(where = 'ey is NULL and bgy <= %s' % jahr, order = 'lnr')
            jghl = Jugendhilfestatistik2007List(where = 'ey is NULL and jahr = %s' % jahr, order = 'lnr')
        else:
            where = 'ey=%s' % jahr
            where_fall = " and fall.zday > 0"
            where_kein_fall = " and fall_id IS NULL"
            join = [('fall', 'jghstat07.fall_id=fall.id')]
            jghl = (Jugendhilfestatistik2007List(where=where+where_fall, join=join) +
                    Jugendhilfestatistik2007List(where=where+where_kein_fall, join=join))
            jghl.sort('lnr')
        if jahr >= 2012:
            check_gef8a(jghl)
    else:
        jghl = JugendhilfestatistikList (where = 'ey =  %s' % jahr, order = 'lnr')
        get_datensatz = get_datensatz_bis_2006
    daten_saetze = []
    log_daten_saetze = []
    log_header = """Erzeugen der Datei mit den Datenstzen fr die Jugendhilfestatistik fr %s
%s
Erstellt am: %s
Datei: %s
Anzahl der Datenstze: %s

Datenstze (vorangestellt zustzlich Fallnummer und Mitarbeiterkennung):

""" % (jahr,
       andauernd and "(andauernde Flle)" or '',
       #time.strftime("%c", time.localtime(time.time())),
       today(),
       get_export_datei_name(jahr, False, andauernd),
       len(jghl))
    for record in jghl:
        daten_satz, log_daten_satz = get_datensatz(record)
        daten_saetze.append(daten_satz)
        log_daten_saetze.append(log_daten_satz)
    return ''.join(daten_saetze), log_header + ''.join(log_daten_saetze)

def get_export_datei_name(jahr, log, andauernd):
    log = log and '_log' or ''
    t = today()
    andauernd = andauernd and (
        "_am_%04d-%02d-%02d_andauernd" % (t.year, t.month, t.day)
        ) or ''
    name = "jgh_%s%s%s.txt" % (jahr, andauernd, log)
    return name

def get_datensatz_bis_2006(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz fr einen Fall

    Zurckgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.
    """
    l = [
      check_code(r['rbz__code'], 1),
      check_code(r['kr__code'][1:3], 2),
      check_code(r['gm__code'], 3),
      check_code(r['gmt__code'], 3),
      "%05d" % r['lnr'],
      check_code(r['traeg__code'], 1),
      "%02d" % r['bgm'],
      "%d" % r['bgy'],
      "%02d" % r['em'],
      "%d" % r['ey'],
      check_code(r['bgr__code'], 1),
      check_code(r['gs__code'], 1),
      check_code(r['ag__code'], 1),
      check_code(r['fs__code'], 2),
      check_code(r['hke__code'], 1),
      check_gsa(r['gsa'], r['gsu__code']),
      check_code(r['zm__code'], 1),
      check_code(r['ba0__code'], 1, '0'),
      check_code(r['ba1__code'], 1, '0'),
      check_code(r['ba2__code'], 1, '0'),
      check_code(r['ba3__code'], 1, '0'),
      check_code(r['ba4__code'], 1, '0'),
      check_code(r['ba5__code'], 1, '0'),
      check_code(r['ba6__code'], 1, '0'),
      check_code(r['ba7__code'], 1, '0'),
      check_code(r['ba8__code'], 1, '0'),
      check_code(r['ba9__code'], 1, '0'),
      check_code(r['schw__code'], 1),
      check_code(r['fbe0__code'], 1, '0'),
      check_code(r['fbe1__code'], 1, '0'),
      check_code(r['fbe2__code'], 1, '0'),
      check_code(r['fbe3__code'], 1, '0'),
      ' '*28,
    ]
    daten_satz = ''.join(l)
    assert len(daten_satz) == 80
    log_daten_satz = "%-10s%-10s%s\r\n" % (r['fall_fn'], r['mit_id__ben'], daten_satz[:54])
    return daten_satz, log_daten_satz
    
def get_datensatz_ab_2007(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz fr einen Fall

    Zurckgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.
    """
    gs = config.JGH_GEMEINDESCHLUESSEL
    if gs == 'blank':
        gs = '   '
    elif not gs:
        gs = '000'
    d = [' ']*210      # Datensatz
    _set(d, 1, 1, 'A')
    _set(d, 2, 3, int(r['land__code']), "%02d")
    #_set(d, 4, 4, '0') # Regierungsbezirk jetzt beim Kreis mit drin
    _set(d, 4, 6, int(r['kr__code']), "%03d")
    _set(d, 7, 9, gs) # Wird wohl nicht ausgewertet??
    _set(d, 10, 15, int(r['einrnr__code']), "%06d")
    _set(d, 16, 20, r['lnr'], '%05d')
    _set(d, 21, 22, r['bgm'], '%02d')
    _set(d, 23, 26, r['bgy'], '%04d')
    _set(d, 27, 27, '%1s' % r['zustw'])
    _set(d, 28, 29, r['hilf_art__code'])
    _set(d, 30, 31, r['hilf_ort__code'])
    _set(d, 32, 33, r['traeger__code'])
    _set(d, 34, 34, r['gs__code'])
    _set(d, 35, 36, r['gem'], '%02d')
    _set(d, 37, 40, r['gey'], '%04d')
    _set(d, 41, 112, ' '*(112-41+1)) # keine andere Kinder
    _set(d, 113, 114, r['aort_vor__code'])
    _set(d, 115, 115, r['sit_fam__code'])
    _set(d, 116, 116, r['ausl_her__code'], None, {'0': ' '})
    _set(d, 117, 117, r['vor_dt__code'], None, {'0': ' '})
    _set(d, 118, 118, r['wirt_sit__code'], None, {'0': ' '})
    _set(d, 119, 119, r['aip__code'])
    _set(d, 120, 120, r['ees__code'])
    _set(d, 121, 121, r['va52__code'])
    _set(d, 122, 122, r['rgu__code'])
    _set(d, 123, 123, r['hda__code'])
    _set(d, 124, 126, r['nbkakt'], '%03d', {None: ' '*3})
    _set(d, 127, 130, ' '*4) # irrelevant fr Erziehungsberatung
    _set(d, 131, 132, r['gr1__code'])
    _set(d, 133, 134, r['gr2__code'], None, {None: ' '*2})
    _set(d, 135, 136, r['gr3__code'], None, {None: ' '*2})
    _set(d, 137, 138, r['em'], '%02d', {None: ' '*2})
    _set(d, 139, 142, r['ey'], '%04d', {None: ' '*4})
    _set(d, 143, 145, r['nbkges'], '%03d', {None: ' '*3})
    _set(d, 146, 146, r['lbk6m__code'], None, {None: ' '})
    _set(d, 147, 150, ' '*4) # irrelevant fr Erziehungsberatung
    _set(d, 151, 152, r['grende__code'], None, {None: ' '*2})
    _set(d, 153, 154, r['aort_nac__code'], None, {None: ' '*2})
    _set(d, 155, 155, r['unh__code'], None, {None: ' '})
    ags = plz = ort = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(r)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        if ags:
            plz = ort = '' # plz, ort nur wenn Gemeindeschlssel unbekannt ist.
    _set(d, 156, 163, ags, '%8s', {None: ' '*8}) # Gemeindeschlssel fr Orte auerhalb des Kreises
    _set(d, 164, 168, plz, '%5s', {None: ' '*5}) # plz ausserhalb
    _set(d, 169, 208, ort, '%-40s', {None: ' '*40}) # ort ausserhalb
    if r['jahr'] >= 2012:
        _set(d, 209, 209, r['gef8a__code'])
        _set(d, 210, 211, '\r\n') # neu: newline zwischen den Datenstzen
        datensatz = ''.join(d)
        assert len(datensatz) == 211
    else:
        _set(d, 209, 210, '\r\n') # neu: newline zwischen den Datenstzen
        datensatz = ''.join(d)
        assert len(datensatz) == 210
    if datensatz[122] == '1': # Hilfe dauert an
        assert datensatz[137:155] == ' '*18
    else:
        assert datensatz[123:126] == ' '*3
    log_datensatz = "%-14s%-10s%s" % (r['fall_fn'], r['mit_id__ben'], datensatz)
    return datensatz, log_datensatz

def get_datensatz_ab_2016(r):
    """Erzeugt einen Jugendhilfestatistikdatensatz fr einen Fall

    Zurckgegeben wird der eigentliche Datensatz sowie ein Kontrolldatensatz.

    Gendert ab 2016: 20-stellige Kennnummer von Position 21 bis 40 eingefgt.
                      Alle folgende Positionen verschieben sich daher um 20 Stellen.
                      Als Kennnummer tragen wir die Fallnummer ein. Dient nur zur Identifikation
                      eines Falles fr evt. Rckfragen, was vorher die Laufende Nummer leistete,
                      die es allerdings auch noch immer gibt.
    """
    gs = config.JGH_GEMEINDESCHLUESSEL
    if gs == 'blank':
        gs = '   '
    elif not gs:
        gs = '000'
    d = [' ']*210      # Datensatz
    _set(d, 1, 1, 'A')
    _set(d, 2, 3, int(r['land__code']), "%02d")
    #_set(d, 4, 4, '0') # Regierungsbezirk jetzt beim Kreis mit drin
    _set(d, 4, 6, int(r['kr__code']), "%03d")
    _set(d, 7, 9, gs) # Wird wohl nicht ausgewertet??
    _set(d, 10, 15, int(r['einrnr__code']), "%06d")
    _set(d, 16, 20, r['lnr'], '%05d')
    _set(d, 21, 40, r['fall_fn'], '%-20s') # Neu 2016

    _set(d, 21+20, 22+20, r['bgm'], '%02d')
    _set(d, 23+20, 26+20, r['bgy'], '%04d')
    _set(d, 27+20, 27+20, '%1s' % r['zustw'])
    _set(d, 28+20, 29+20, r['hilf_art__code'])
    _set(d, 30+20, 31+20, r['hilf_ort__code'])
    _set(d, 32+20, 33+20, r['traeger__code'])
    _set(d, 34+20, 34+20, r['gs__code'])
    _set(d, 35+20, 36+20, r['gem'], '%02d')
    _set(d, 37+20, 40+20, r['gey'], '%04d')
    _set(d, 41+20, 112+20, ' '*(112-41+1)) # keine andere Kinder
    _set(d, 113+20, 114+20, r['aort_vor__code'])
    _set(d, 115+20, 115+20, r['sit_fam__code'])
    _set(d, 116+20, 116+20, r['ausl_her__code'], None, {'0': ' '})
    _set(d, 117+20, 117+20, r['vor_dt__code'], None, {'0': ' '})
    _set(d, 118+20, 118+20, r['wirt_sit__code'], None, {'0': ' '})
    _set(d, 119+20, 119+20, r['aip__code'])
    _set(d, 120+20, 120+20, r['ees__code'])
    _set(d, 121+20, 121+20, r['va52__code'])
    _set(d, 122+20, 122+20, r['rgu__code'])
    _set(d, 123+20, 123+20, r['hda__code'])
    _set(d, 124+20, 126+20, r['nbkakt'], '%03d', {None: ' '*3})
    _set(d, 127+20, 130+20, ' '*4) # irrelevant fr Erziehungsberatung
    _set(d, 131+20, 132+20, r['gr1__code'])
    _set(d, 133+20, 134+20, r['gr2__code'], None, {None: ' '*2})
    _set(d, 135+20, 136+20, r['gr3__code'], None, {None: ' '*2})
    _set(d, 137+20, 138+20, r['em'], '%02d', {None: ' '*2})
    _set(d, 139+20, 142+20, r['ey'], '%04d', {None: ' '*4})
    _set(d, 143+20, 145+20, r['nbkges'], '%03d', {None: ' '*3})
    _set(d, 146+20, 146+20, r['lbk6m__code'], None, {None: ' '})
    _set(d, 147+20, 150+20, ' '*4) # irrelevant fr Erziehungsberatung
    _set(d, 151+20, 152+20, r['grende__code'], None, {None: ' '*2})
    _set(d, 153+20, 154+20, r['aort_nac__code'], None, {None: ' '*2})
    _set(d, 155+20, 155+20, r['unh__code'], None, {None: ' '})
    ags = plz = ort = ''
    ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(r)
    if ags_plz_ort:
        ags, plz, ort = ags_plz_ort
        if ags:
            plz = ort = '' # plz, ort nur wenn Gemeindeschlssel unbekannt ist.
    _set(d, 156+20, 163+20, ags, '%8s', {None: ' '*8}) # Gemeindeschlssel fr Orte auerhalb des Kreises
    _set(d, 164+20, 168+20, plz, '%5s', {None: ' '*5}) # plz ausserhalb
    _set(d, 169+20, 208+20, ort, '%-40s', {None: ' '*40}) # ort ausserhalb
    _set(d, 209+20, 209+20, r['gef8a__code'])
    _set(d, 210+20, 211+20, '\r\n') # neu: newline zwischen den Datenstzen
    datensatz = ''.join(d)
    assert len(datensatz) == 211+20
    if datensatz[122+20] == '1': # Hilfe dauert an
        assert datensatz[137+20:155+20] == ' '*18
    else:
        assert datensatz[123+20:126+20] == ' '*3
    log_datensatz = "%-14s%-10s%s" % (r['fall_fn'], r['mit_id__ben'], datensatz)
    return datensatz, log_datensatz

def _set(vector, von, bis, val, template=None, substitutions=None):
    res = None
    if substitutions:
        s = substitutions.get(val)
        if s != None:
            res = s
    if res == None:
        if template:
            res = template % val
        else:
            res = val
    #print 'VAL', val
    assert len(res) == (bis + 1 - von), 'Fehler beim Export der Bundesstatistik. Spalte %s-%s' % (von, bis)
    vector[von-1:bis] = list(res)

def check_code(str, length, leer_code = None):
    assert len(str) == length
    if not leer_code is None and str == leer_code:
        str = ' '*length
    return str
    
def check_gsa(gsa, gsu):
    """Falls das gsu-Feld 1 ist (Geschwisteranzahl unbekannt), wird das
    gsa Feld immer mit Leerzeichen gefllt. Sonst wird die Zahl aus gsa
    bernommen. """
    if gsu == '1':
        return '  1'
    else:
        return "%02d " % gsa
        

if __name__ == '__main__':
    if len(sys.argv) < 2 or sys.argv[1] in ('-h', '-help', '--help'):
        print "Aufruf: python jghexport.py <jahr> [andauernd]"
        sys.exit(1)
    try:
        jahr = sys.argv[1]
        jahr = int(jahr)
        assert jahr > 1995 and jahr < 2500
    except:
        print "***Fehler: Kein gltiges Jahr: '%s'" % jahr
        sys.exit(1)
    try:
        andauernd = sys.argv[2]
        if andauernd.startswith('a'):
            andauernd = True
        else:
            print "***Fehler: Zweiter Parameter muss mit 'a' (fuer andauernd) beginnen"
            sys.exit(1)
    except IndexError:
        andauernd = False
    print
    print "Export der Jugendhilfestatistik fuer Instanz '%s'" % init.INSTANCE_NAME
    print "   (Pfad %s)" % init.INSTANCE_HOME
    print
    print "Jahr: %s  (%s)" % (jahr, andauernd and "andauernde Faelle" or "abgeschlossene Faelle" )
    print
    from ebkus.db.sql import opendb
    opendb()
    daten_saetze, log_daten_saetze = jghexport(jahr, andauernd)

    datei_name = get_export_datei_name(jahr, False, andauernd)
    f = open(datei_name, "w")
    f.write(daten_saetze)
    f.close()
    print "ausgegeben: %s" % datei_name 

    log_datei_name = get_export_datei_name(jahr, True, andauernd)
    f = open(log_datei_name, "w")
    f.write(log_daten_saetze)
    f.close()
    print "ausgegeben: %s" % log_datei_name
    print
    
    
    
    
    
